<?php

	class Icwoorok2Controller
	{
		private static $bInitiated = false;

		public static function init()
		{
			if(self::$bInitiated)
			{
				return;
			}

			// Load methods
			add_action('woocommerce_payment_gateways', array(__CLASS__, 'getGateways'));

			// Load settings
			add_filter('woocommerce_payment_gateways_settings', array(__CLASS__, 'addPspSettings'));

			// Set hooks for the return and webhook call
			add_action('woocommerce_api_wc_omnikassa_gateway_return', array(__CLASS__, 'doReturn'));
			add_action('woocommerce_api_wc_omnikassa_gateway_notify', array(__CLASS__, 'doNotify'));

			// Mark plugin initiated
			self::$bInitiated = true;
		}


		public static function getGateways($aDefaultGateways)
		{
			// Load our gateways
			$sFolderPath = ICWOOROK_ROOT_PATH . DIRECTORY_SEPARATOR . 'gateways';
			icwoorok2_loadFilesFromFolder($sFolderPath);

			$aPaymentGateways = array(
				'icwoorok2_bancontact',
				'icwoorok2_ideal',
				'icwoorok2_afterpay',
				'icwoorok2_creditcard',
				'icwoorok2_paypal',
			);

			$aDefaultGateways = array_merge($aDefaultGateways, $aPaymentGateways);

			return $aDefaultGateways;
		}

		public static function addPspSettings($aSettings)
		{
			$aAlteredSettings = array();

			$sWebhookUrl = add_query_arg('wc-api', 'Wc_Omnikassa_Gateway_Notify', home_url('/'));

			$aGlobalSettings = array();
			$aGlobalSettings[] = array(
				'name' => __('Rabo OmniKassa 2.0 - Merchant settings', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'title',
				'desc' => __('IMPORTANT: To process transactions properly, please enter the following webhook URL on your Rabo OmniKassa dashboard:', 'ic-woo-rabo-omnikassa-2') . ' <a href="' . $sWebhookUrl . '">' . $sWebhookUrl . '</a></p>',
			);
			$aGlobalSettings[] = array(
				'name' => __('Refresh Token', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'textarea',
				'css' => 'height:200px;font-size:12px;',
				'desc' => '<p>' . __('The Refresh token can be found on the Rabo OmniKassa 2.0 dashboard.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_refresh_token',
			);
			$aGlobalSettings[] = array(
				'name' => __('Signing Key', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'text',
				'css' => 'font-size:12px;',
				'desc' => '<p>' . __('The Signing Key token can be found on the Rabo OmniKassa 2.0 dashboard.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_signing_key',
			);
			$aGlobalSettings[] = array(
				'name' => __('Sandbox environment', 'ic-woo-rabo-omnikassa-2'),
				'type' => 'checkbox',
				'label' => __('Enable the Sandbox mode', 'ic-woo-rabo-omnikassa-2'),
				'default' => 'yes',
				'desc' => '<p>' . __('Please make sure you enter the sandbox credentials if checked.', 'ic-woo-rabo-omnikassa-2') . '</p>',
				'id' => 'icwoorok2_sandbox',
			);
			$aGlobalSettings[] = array(
				'type' => 'sectionend',
			);

			foreach($aSettings as $aSetting)
			{
				if(isset($aSetting['id']) && $aSetting['id'] == 'payment_gateways_options' && $aSetting['type'] != 'sectionend')
				{
					$aAlteredSettings = array_merge($aAlteredSettings, $aGlobalSettings);
				}

				$aAlteredSettings[] = $aSetting;
			}

			return $aAlteredSettings;
		}

		public static function doReturn()
		{
			global $woocommerce;

			if(empty($_GET['status']) && empty($_GET['order_id']) && empty($_GET['signature']) && empty($_GET['real_id']))
			{
				wp_redirect($woocommerce->cart->get_cart_url());
			}
			else
			{
				$sMerchantOrderId = icwoorok2_getUrlParam('order_id');
				$sOmniKassaStatus = icwoorok2_getUrlParam('status');
				$sOmniKassaSignature = icwoorok2_getUrlParam('signature');
				$sOrderId = icwoorok2_getUrlParam('real_id', true, true);
				$bUtmOverride = array_key_exists('utm_nooverride', $_GET);

				$oOrder = wc_get_order($sOrderId);
				$sPaymentMethod = $oOrder->get_payment_method_title();
				$sTransactionId = $oOrder->get_transaction_id();

				if(strcasecmp($sOmniKassaStatus, 'COMPLETED') === 0)
				{
					$sHashString = $sMerchantOrderId . ',' . $sOmniKassaStatus;
					$sHash = hash_hmac('sha512', $sHashString, base64_decode(get_option('icwoorok2_signing_key')));

					if(strcasecmp($sOmniKassaSignature, $sHash) === 0)
					{
						$sReturnUrl = $oOrder->get_checkout_order_received_url();


						$oOrder->add_order_note(__('Status received from Customer Return:', 'ic-woo-rabo-omnikassa-2') . $sOmniKassaStatus . ' (Order not updated, check OmniKassa dashboard for status.) ' . __('Payment-method: ', 'ic-woo-rabo-omnikassa-2') . $sPaymentMethod);
						wp_redirect($sReturnUrl . ($bUtmOverride ? '&utm_nooverride=1' : ''));
					}
					else
					{
						wp_redirect($woocommerce->cart->get_cart_url() . ($bUtmOverride ? '&utm_nooverride=1' : ''));
					}
				}
				else
				{
					$oOrder->add_order_note(__('Status received from Customer Return:', 'ic-woo-rabo-omnikassa-2') . $sOmniKassaStatus . ' (Order not updated, check OmniKassa dashboard for status.) ' . __('Payment-method: ', 'ic-woo-rabo-omnikassa-2') . $sPaymentMethod);
					wp_redirect($woocommerce->cart->get_cart_url() . ($bUtmOverride ? '&utm_nooverride=1' : ''));
				}
			}
		}

		public static function doNotify($return = false)
		{
			global $woocommerce;
			$sHtml = '';

			// Build the notify from the Rabobank OmniKassa
			$sJsonData = @file_get_contents('php://input');

			if(empty($sJsonData))
			{
				$sHtml .= '<p>Invalid notification call.</p>';
			}
			else
			{
				$aPostData = json_decode($sJsonData, true);

				if(!empty($aPostData['authentication']) && !empty($aPostData['expiry']) && !empty($aPostData['eventName']) && !empty($aPostData['poiId']) && !empty($aPostData['signature']))
				{
					$sAuthToken = sanitize_text_field($aPostData['authentication']);
					$sExpiryString = sanitize_text_field($aPostData['expiry']);
					$sEventnameString = sanitize_text_field($aPostData['eventName']);
					$sPoiIdString = sanitize_text_field($aPostData['poiId']);
					$sSignatureString = sanitize_text_field($aPostData['signature']);

					// Check the signature that was sent with the data
					$sHashString = $sAuthToken . ',' . $sExpiryString . ',' . $sEventnameString . ',' . $sPoiIdString;

					$sHash = hash_hmac('sha512', $sHashString, base64_decode(get_option('icwoorok2_signing_key')));

					if(strcasecmp($sSignatureString, $sHash) === 0)
					{
						$sEventName = 'merchant.order.status.changed'; //$aPostData['eventName'];
						$bNextOrder = false;

						$sTestMode = get_option('icwoorok2_sandbox');
						$bSandboxMode = (strcmp($sTestMode, 'yes') === 0);

						$sApiUrl = 'https://betalen.rabobank.nl/omnikassa-api' . ($bSandboxMode ? '-sandbox' : '') . '/order/server/api/events/results/merchant.order.status.changed';

						$aResult = array();

						do
						{
							$aArguments = array();
							$aArguments['body'] = '';
							$aArguments['timeout'] = '30';
							$aArguments['redirection'] = '5';
							$aArguments['httpversion'] = '1.1';
							$aArguments['blocking'] = true;
							$aArguments['headers'] = array('Expect' => '', 'Authorization'=> 'Bearer ' . $sAuthToken);
							$aArguments['cookies'] = array();

							$oResponse = wp_remote_get($sApiUrl, $aArguments);
							$sResponse = wp_remote_retrieve_body($oResponse);

							// error_log($sResponse);

							$aResult = json_decode($sResponse, true);
							
							if(sizeof($aResult))
							{
								$sSignature = $aResult['signature'];
								$bMoreOrders = $aResult['moreOrderResultsAvailable'];

								if($bMoreOrders)
								{
									$sMoreOrdersAvailable = 'true';
								}
								else
								{
									$sMoreOrdersAvailable = 'false';

								}

								$sHashString = $sMoreOrdersAvailable . ',';

								// Validate total signature
								foreach($aResult['orderResults'] as $aTransaction)
								{
									// Setup total hash string
									$sTransactionString = $aTransaction['merchantOrderId'] . ',' . $aTransaction['omnikassaOrderId'] . ',' . $aTransaction['poiId'] . ',' . $aTransaction['orderStatus'] . ',' . $aTransaction['orderStatusDateTime'] . ',' . $aTransaction['errorCode'] . ',' . $aTransaction['paidAmount']['currency'] . ',' . $aTransaction['paidAmount']['amount'] . ',' . $aTransaction['totalAmount']['currency'] . ',' . $aTransaction['totalAmount']['amount'];

									$sHashString .= $sTransactionString . ',';
								}

								// Cut off last comma
								$sHashString = substr($sHashString, 0, -1);
								$sHash = hash_hmac('sha512', $sHashString, base64_decode(get_option('icwoorok2_signing_key')));

								if(strcasecmp($sSignature, $sHash) === 0)
								{
									foreach($aResult['orderResults'] as $aTransaction)
									{
										$sMerchantOrderId = $aTransaction['merchantOrderId'];
										$sOmniKassaStatus = $aTransaction['orderStatus'];
										$sTransactionId = $aTransaction['omnikassaOrderId'];

										// Get order by Merchant Order ID
										$oOrder = wc_get_order($sMerchantOrderId);

										$sMessage = __('Status received from Rabo OmniKassa: ' . $sOmniKassaStatus . ' for transaction ID:<br>' . $sTransactionId, 'ic-woo-rabo-omnikassa-2');

										if(strcasecmp($sOmniKassaStatus, 'COMPLETED') === 0)
										{
											$oOrder->add_order_note($sMessage);
											$oOrder->payment_complete($sTransactionId);
										}
										elseif(strcasecmp($sOmniKassaStatus, 'CANCELLED') === 0)
										{
											// Woocommerce cancels automatically
											$oOrder->add_order_note($sMessage);
										}
										elseif(strcasecmp($sOmniKassaStatus, 'EXPIRED') === 0)
										{
											// $oOrder->add_order_note('Status received from Rabo OmniKassa: ' . $sOmnikassaStatus);
											$oOrder->update_status('cancelled', $sMessage);
										}
										else // pending
										{
											$oOrder->add_order_note($sMessage);
										}
									}
								}
							}
						}
						while($bMoreOrders);
					}
				}
				else
				{
					$sHtml .= '<p>Invalid notification call.</p>';
				}
			}

			echo $sHtml;
			exit;
		}
	}


?>